# Copyright 2014 Hewlett-Packard Development Company, L.P.
#
# Author: Endre Karlson <endre.karlson@hpe.com>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from designate.objects import base
from designate.objects import fields


@base.DesignateRegistry.register
class ZoneAttribute(base.DictObjectMixin, base.PersistentObjectMixin,
                    base.DesignateObject):
    fields = {
        'zone_id': fields.UUIDFields(nullable=True),
        'key': fields.StringFields(maxLength=50, nullable=False),
        'value': fields.StringFields(maxLength=50, nullable=False)
    }

    STRING_KEYS = [
        'id', 'key', 'value', 'zone_id'
    ]


@base.DesignateRegistry.register
class ZoneAttributeList(base.AttributeListObjectMixin, base.DesignateObject):
    LIST_ITEM_TYPE = ZoneAttribute

    fields = {
        'objects': fields.ListOfObjectsField('ZoneAttribute'),
    }
